package s4;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferStrategy;
import java.util.Random;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import s3.Animation;

public class SpriteSample2 extends JFrame {

	private Image bgImage;
	private Sprite[] sprites;

	private final static int SPRITE_NUM = 5;
	private final static int FADE_TIME = 2000;
	private final static int DEMO_TIME = 1000*10;





	public void prepare(){
		bgImage = loadImage("image/bg.jpg");
		
		Image image1 = loadImage("image/img1.gif");
		Image image2 = loadImage("image/img2.gif");
		Image image3 = loadImage("image/img3.gif");

		sprites = new Sprite[SPRITE_NUM];
		Random random = new Random();
		for(int i=0; i<SPRITE_NUM; i++){
			Animation anim = new Animation();
			anim.addFrame(image1, 500);
			anim.addFrame(image2, 500);
			anim.addFrame(image3, 100+random.nextInt(500));
			sprites[i] = new Sprite(anim);
			sprites[i].setVelocityX(random.nextFloat()/2);
			sprites[i].setVelocityY(random.nextFloat()/4);
		}
	}
	
	public void loopAnim(){
		
		createBufferStrategy(2);
		
		long startTime = System.currentTimeMillis();
		long nowTime = startTime;
		
		while(nowTime - startTime < DEMO_TIME){

			long elapsedTime = System.currentTimeMillis() - nowTime;
			nowTime += elapsedTime;

			updateSprites(elapsedTime);
			
			BufferStrategy strategy = getBufferStrategy();
			Graphics g = strategy.getDrawGraphics();

			g.drawImage(bgImage, 0, 0, null);
			drawSprites((Graphics2D)g);
			drawFade((Graphics2D)g, nowTime-startTime);
			
			g.dispose();
			strategy.show();
			
			try {
				Thread.sleep(20);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}

	public void drawFade(Graphics2D g, long currTime) {
		long time = 0;
		if (currTime <= FADE_TIME) {
			time = FADE_TIME - currTime;
		}
		else if (currTime > DEMO_TIME - FADE_TIME) {
			time = FADE_TIME - DEMO_TIME + currTime;
		}
		else {
			return;
		}

		byte numBars = 8;
		int barHeight = getHeight() / numBars;
		int blackHeight = (int)(time * barHeight / FADE_TIME);

		g.setColor(Color.black);
		for (int i = 0; i < numBars; i++) {
			int y = i * barHeight + (barHeight - blackHeight) / 2;
			g.fillRect(0, y, getWidth(), blackHeight);
		}

	}



	private void updateSprites(long elapsedTime){
		
		for(int i=0; i<SPRITE_NUM; i++){
			Sprite sprite = sprites[i];
		
			if(sprite.getX() < 0){
				sprite.setVelocityX(Math.abs(sprite.getVelocityX()));
			}else if(sprite.getX() > 600){
				sprite.setVelocityX(-Math.abs(sprite.getVelocityX()));			
			}
			
			if(sprite.getY() < 0){
				sprite.setVelocityY(Math.abs(sprite.getVelocityY()));
			}else if(sprite.getY() > 400){
				sprite.setVelocityY(-Math.abs(sprite.getVelocityY()));
			}
			
			sprite.update(elapsedTime);
		}
	}
	
	
	private void drawSprites(Graphics2D g){
		
		AffineTransform transform = new AffineTransform();


		for(int i=0; i<SPRITE_NUM; i++){
			Sprite sprite = sprites[i];

			transform.setToTranslation(sprite.getX(),
				sprite.getY());
			
			if(sprite.getVelocityX() < 0){
				transform.scale(-1, 1);
				transform.translate(-sprite.getWidth(), 0);
			}
			
			g.drawImage(sprite.getImage(), transform, null);
		}
	}



	
	private Image loadImage(String path){
		return new ImageIcon(path).getImage();
	}
	

	public static void main(String[] args) {
		SpriteSample2 frame = new SpriteSample2();

		frame.prepare();

		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(640, 480);
		frame.setVisible(true);

		frame.loopAnim();
	}
	



	
}
